// ICC file to gamma ramp routines are taken from xcalib
// http://xcalib.sourceforge.net/


#include <stdio.h>
#include <tchar.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qfileinfo.h>

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>


# include <windows.h>
# include <wingdi.h>


#include <math.h>

/* the 4-byte marker for the vcgt-Tag */
#define VCGT_TAG     0x76636774L
#define MLUT_TAG     0x6d4c5554L

#ifndef XCALIB_VERSION
# define XCALIB_VERSION "version unknown (>0.5)"
#endif

/* a limit to check the table sizes (of corrupted profiles) */
#ifndef MAX_TABLE_SIZE
# define MAX_TABLE_SIZE   2e10
#endif

#ifdef WIN32GDI
# define u_int16_t  WORD
#endif

# define BE_INT(a)    ((a)[3]+((a)[2]<<8)+((a)[1]<<16) +((a)[0]<<24))
# define BE_SHORT(a)  ((a)[1]+((a)[0]<<8))

/* internal state struct */
struct xcalib_state_t {
	unsigned int verbose;
	float redGamma;
	float redMin;
	float redMax;
	float greenGamma;
	float greenMin;
	float greenMax;
	float blueGamma;
	float blueMin;
	float blueMax;
	float gamma_cor;
} xcalib_state = {0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0, 1.0};

static unsigned int monitorSearchIndex = 0;
static HDC monitorDC = 0;

BOOL CALLBACK MonitorEnumProc (HMONITOR monitor, HDC hdc, LPRECT rect, LPARAM data)
{
	MONITORINFOEX monitorInfo;

	if(monitorSearchIndex++ != (unsigned int)data)
		return TRUE; /* continue enumeration */

	monitorInfo.cbSize = sizeof(monitorInfo);
	if(GetMonitorInfo(monitor, (LPMONITORINFO)&monitorInfo) )
		monitorDC = CreateDC(NULL, monitorInfo.szDevice, NULL, NULL);

	return FALSE;  /* stop enumeration */
}

HDC FindMonitor(int index)
{
	if(index == -1)
		return GetDC(NULL); /* return primary display context */

	monitorSearchIndex = 0;
	monitorDC = 0;
	EnumDisplayMonitors(NULL, NULL, MonitorEnumProc, index);
	return monitorDC;
}

/*
* FUNCTION read_vcgt_internal
*
* this is a parser for the vcgt tag of ICC profiles which tries to
* resemble most of the functionality of Graeme Gill's icclib.
*
* returns
* -1: file could not be read
* 0: file okay but doesn't contain vcgt or MLUT tag
* 1: success
*/
int	read_vcgt_internal(const char * filename, u_int16_t * rRamp, u_int16_t * gRamp,	u_int16_t * bRamp, unsigned int nEntries)
{
	FILE * fp;
	unsigned int bytesRead;
	unsigned int numTags=0;
	unsigned int tagName=0;
	unsigned int tagOffset=0;
	unsigned int tagSize=0;
	unsigned char cTmp[4];
	unsigned int uTmp;
	unsigned int gammaType;

	signed int retVal=0;

	u_int16_t * redRamp = NULL, * greenRamp = NULL, * blueRamp = NULL;
	unsigned int ratio=0;
	/* formula */
	float rGamma, rMin, rMax;
	float gGamma, gMin, gMax;
	float bGamma, bMin, bMax;
	int i=0;
	/* table */
	unsigned int numChannels=0;
	unsigned int numEntries=0;
	unsigned int entrySize=0;
	int j=0;

	if(filename) {
		fp = fopen(filename, "rb");
		if(!fp)
			return -1; /* file can not be opened */
	} else
		return -1; /* filename char pointer not valid */
	/* skip header */
	if(fseek(fp, 0+128, SEEK_SET))
	{
		fclose(fp);
		return  -1;
	}
	/* check num of tags in current profile */
	bytesRead = fread(cTmp, 1, 4, fp);
	numTags = BE_INT(cTmp);
	for(i=0; i<numTags; i++) {
		bytesRead = fread(cTmp, 1, 4, fp);
		tagName = BE_INT(cTmp);
		bytesRead = fread(cTmp, 1, 4, fp);
		tagOffset = BE_INT(cTmp); 
		bytesRead = fread(cTmp, 1, 4, fp);
		tagSize = BE_INT(cTmp);
		if(!bytesRead)
			break;
		if(tagName == MLUT_TAG)
		{
			if(fseek(fp, 0+tagOffset, SEEK_SET))
				break;
			//message("mLUT found (Profile Mechanic)\n");
			redRamp = (unsigned short *) malloc ((256) * sizeof (unsigned short));
			greenRamp = (unsigned short *) malloc ((256) * sizeof (unsigned short));
			blueRamp = (unsigned short *) malloc ((256) * sizeof (unsigned short));
			{
				for(j=0; j<256; j++) {
					bytesRead = fread(cTmp, 1, 2, fp);
					redRamp[j]= BE_SHORT(cTmp);
				}
				for(j=0; j<256; j++) {
					bytesRead = fread(cTmp, 1, 2, fp);
					greenRamp[j]= BE_SHORT(cTmp);
				}
				for(j=0; j<256; j++) {
					bytesRead = fread(cTmp, 1, 2, fp);
					blueRamp[j]= BE_SHORT(cTmp);
				}
			}
			/* simply copy values to the external table (and leave some values out if table size < 256) */
			ratio = (unsigned int)(256 / (nEntries));
			for(j=0; j<nEntries; j++) {
				rRamp[j] = redRamp[ratio*j];
				gRamp[j] = greenRamp[ratio*j];
				bRamp[j] = blueRamp[ratio*j];
			}
			free(redRamp);
			free(greenRamp);
			free(blueRamp);
			retVal = 1;
			break;
		}
		if(tagName == VCGT_TAG)
		{
			fseek(fp, 0+tagOffset, SEEK_SET);
			//message("vcgt found\n");
			bytesRead = fread(cTmp, 1, 4, fp);
			tagName = BE_INT(cTmp);
			if(tagName != VCGT_TAG)
			{
				//warning("invalid content of table vcgt, starting with %x",
				//      tagName);
				break;
			}
			bytesRead = fread(cTmp, 1, 4, fp);
			bytesRead = fread(cTmp, 1, 4, fp);
			gammaType = BE_INT(cTmp);
			/* VideoCardGammaFormula */
			if(gammaType==1) {
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				rGamma = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				rMin = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				rMax = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				gGamma = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				gMin = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				gMax = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				bGamma = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				bMin = (float)uTmp/65536.0;
				bytesRead = fread(cTmp, 1, 4, fp);
				uTmp = BE_INT(cTmp);
				bMax = (float)uTmp/65536.0;

				if(rGamma > 5.0 || gGamma > 5.0 || bGamma > 5.0)
				{
					//warning("Gamma values out of range (> 5.0): \nR: %f \tG: %f \t B: %f",
					//      rGamma, gGamma, bGamma);
					break;
				}
				if(rMin >= 1.0 || gMin >= 1.0 || bMin >= 1.0)
				{
					//warning("Gamma lower limit out of range (>= 1.0): \nRMin: %f \tGMin: %f \t BMin: %f",
					//      rMin, gMin, bMin);
					break;
				}
				if(rMax > 1.0 || gMax > 1.0 || bMax > 1.0)
				{
					//warning("Gamma upper limit out of range (> 1.0): \nRMax: %f \tGMax: %f \t BMax: %f",
					//      rMax, gMax, bMax);
					break;
				}
				//message("Red:   Gamma %f \tMin %f \tMax %f\n", rGamma, rMin, rMax);
				//message("Green: Gamma %f \tMin %f \tMax %f\n", gGamma, gMin, gMax);
				//message("Blue:  Gamma %f \tMin %f \tMax %f\n", bGamma, bMin, bMax);

				for(j=0; j<nEntries; j++) {
					rRamp[j] = 65536.0 *
						((double) pow ((double) j / (double) (nEntries),
						rGamma * (double) xcalib_state.gamma_cor 
						) * (rMax - rMin) + rMin);
					gRamp[j] = 65536.0 *
						((double) pow ((double) j / (double) (nEntries),
						gGamma * (double) xcalib_state.gamma_cor
						) * (gMax - gMin) + gMin);
					bRamp[j] = 65536.0 *
						((double) pow ((double) j / (double) (nEntries),
						bGamma * (double) xcalib_state.gamma_cor
						) * (bMax - bMin) + bMin);
				}
				retVal = 1;
			}
			/* VideoCardGammaTable */
			else if(gammaType==0) {
				bytesRead = fread(cTmp, 1, 2, fp);
				numChannels = BE_SHORT(cTmp);
				bytesRead = fread(cTmp, 1, 2, fp);
				numEntries = BE_SHORT(cTmp);
				bytesRead = fread(cTmp, 1, 2, fp);
				entrySize = BE_SHORT(cTmp);

				/* work-around for AdobeGamma-Profiles */
				if(tagSize == 1584) {
					entrySize = 2;
					numEntries = 256;
					numChannels = 3;
				}

				//message ("channels:        \t%d\n", numChannels);
				//message ("entry size:      \t%dbits\n",entrySize  * 8);
				//message ("entries/channel: \t%d\n", numEntries);
				//message ("tag size:        \t%d\n", tagSize);

				if(numChannels!=3)          /* assume we have always RGB */
					break;

				/* allocate tables for the file plus one entry for extrapolation */
				redRamp = (unsigned short *) malloc ((numEntries+1) * sizeof (unsigned short));
				greenRamp = (unsigned short *) malloc ((numEntries+1) * sizeof (unsigned short));
				blueRamp = (unsigned short *) malloc ((numEntries+1) * sizeof (unsigned short));
				{		  
					for(j=0; j<numEntries; j++) {
						switch(entrySize) {
						case 1:
							bytesRead = fread(cTmp, 1, 1, fp);
							redRamp[j]= cTmp[0] << 8;
							break;
						case 2:
							bytesRead = fread(cTmp, 1, 2, fp);
							redRamp[j]= BE_SHORT(cTmp);
							break;
						}
					}
					for(j=0; j<numEntries; j++) {
						switch(entrySize) {
						case 1:
							bytesRead = fread(cTmp, 1, 1, fp);
							greenRamp[j]= cTmp[0] << 8;
							break;
						case 2:
							bytesRead = fread(cTmp, 1, 2, fp);
							greenRamp[j]= BE_SHORT(cTmp);
							break;
						}
					}
					for(j=0; j<numEntries; j++) {
						switch(entrySize) {
						case 1:                bytesRead = fread(cTmp, 1, 1, fp);
							blueRamp[j]= cTmp[0] << 8;
							break;
						case 2:
							bytesRead = fread(cTmp, 1, 2, fp);
							blueRamp[j]= BE_SHORT(cTmp);
							break;
						}
					}
				}

				if(numEntries >= nEntries) {
					/* simply subsample if the LUT is smaller than the number of entries in the file */
					ratio = (unsigned int)(numEntries / (nEntries));
					for(j=0; j<nEntries; j++) {
						rRamp[j] = redRamp[ratio*j];
						gRamp[j] = greenRamp[ratio*j];
						bRamp[j] = blueRamp[ratio*j];
					}
				}
				else {
					ratio = (unsigned int)(nEntries / numEntries);
					/* add extrapolated upper limit to the arrays - handle overflow */
					redRamp[numEntries] = (redRamp[numEntries-1] + (redRamp[numEntries-1] - redRamp[numEntries-2])) & 0xffff;
					if(redRamp[numEntries] < 0x4000)
						redRamp[numEntries] = 0xffff;

					greenRamp[numEntries] = (greenRamp[numEntries-1] + (greenRamp[numEntries-1] - greenRamp[numEntries-2])) & 0xffff;
					if(greenRamp[numEntries] < 0x4000)
						greenRamp[numEntries] = 0xffff;

					blueRamp[numEntries] = (blueRamp[numEntries-1] + (blueRamp[numEntries-1] - blueRamp[numEntries-2])) & 0xffff;
					if(blueRamp[numEntries] < 0x4000)
						blueRamp[numEntries] = 0xffff;

					for(j=0; j<numEntries; j++) {
						for(i=0; i<ratio; i++)
						{
							rRamp[j*ratio+i] = (redRamp[j]*(ratio-i) + redRamp[j+1]*(i)) / (ratio);
							gRamp[j*ratio+i] = (greenRamp[j]*(ratio-i) + greenRamp[j+1]*(i)) / (ratio);
							bRamp[j*ratio+i] = (blueRamp[j]*(ratio-i) + blueRamp[j+1]*(i)) / (ratio);
						}
					}
				}
				free(redRamp);
				free(greenRamp);
				free(blueRamp);
				retVal = 1;
			}
			break;
		} /* for all tags */
	}
	fclose(fp);
	return retVal;
}

bool grabAndSaveCurrentProfile(int monitorID, QString destFile)
{
	typedef struct _GAMMARAMP {
		WORD  Red[256];
		WORD  Green[256];
		WORD  Blue[256];
	} GAMMARAMP; 
	GAMMARAMP winGammaRamp;
	HDC hDc = FindMonitor(monitorID);
	if(hDc && GetDeviceGammaRamp(hDc, &winGammaRamp))
	{
		QFile file(destFile);
		if(!file.open(QIODevice::WriteOnly))
		{
			QMessageBox::information(NULL, "CPKeeper","Couldn't write to " + destFile);
			ReleaseDC(0, hDc);
			return false;
		}
		file.write("CPK",3);
		file.write((const char*)&winGammaRamp, sizeof(GAMMARAMP));
		file.close();
	}
	else
	{
		QMessageBox::information(NULL, "CPKeeper","Couldn't grab current gamma ramp for monitor " + QString::number(monitorID));
		return false;
	}
	if(hDc)
		ReleaseDC(0, hDc);
	return true;
}

bool applyCPKFile(int monitorID, QString infile)
{
	typedef struct _GAMMARAMP {
		WORD  Red[256];
		WORD  Green[256];
		WORD  Blue[256];
	} GAMMARAMP; 
	GAMMARAMP winGammaRamp;

	QFileInfo finfo(infile);
	if(finfo.size() != 3 + sizeof(GAMMARAMP))
	{
		return false;
	}
	QFile file(infile);
	if(!file.open(QIODevice::ReadOnly))
	{
		return false;
	}
	file.seek(3);
	file.read((char*)&winGammaRamp, sizeof(GAMMARAMP));
	file.close();
	HDC hDc = FindMonitor(monitorID);
	if(hDc)
	{  
		static bool altern=true;
		if(altern)
			winGammaRamp.Blue[127]++;
		else
			winGammaRamp.Blue[127]--;
		altern = !altern;
		bool ok = SetDeviceGammaRamp(hDc, &winGammaRamp);
		ReleaseDC(NULL, hDc);
		return true;
	}
	return false;
}

int	xcalib (int monitorID, char *profileFile)
{
	if(QString(profileFile).endsWith(".cpk", Qt::CaseInsensitive))
		return applyCPKFile(monitorID, QString(profileFile));
	int fa, nfa;			/* argument we're looking at */
	char in_name[256] = { '\000' };
	char tag_name[40] = { '\000' };
	int verb = 2;
	int search = 0;
	int ecount = 1;		/* Embedded count */
	int offset = 0;		/* Offset to read profile from */
	int found;
	int rv = 0;
	u_int16_t *r_ramp = NULL, *g_ramp = NULL, *b_ramp = NULL;
	int i;
	int clear = 0;
	int alter = 0;
	int donothing = 0;
	int printramps = 0;
	int calcloss = 0;
	int invert = 0;
	int correction = 0;
	u_int16_t tmpRampVal = 0;
	unsigned int r_res, g_res, b_res;
	int screen = -1;

	unsigned int ramp_size = 256;
	unsigned int ramp_scaling;

	char win_default_profile[MAX_PATH+1];
	DWORD win_profile_len;
	typedef struct _GAMMARAMP {
		WORD  Red[256];
		WORD  Green[256];
		WORD  Blue[256];
	} GAMMARAMP; 
	GAMMARAMP winGammaRamp;
	HDC hDc = NULL;

	xcalib_state.verbose = 0;

	for(i=0; i< ramp_size; i++) {
		winGammaRamp.Red[i] = i << 8;
		winGammaRamp.Blue[i] = i << 8;
		winGammaRamp.Green[i] = i << 8;
	}

	screen = monitorID;

	if(!clear || !alter)
	{
		if(strlen(profileFile) < 255)
			strcpy (in_name, profileFile);

	}

	if ((!clear || !alter) && (in_name[0] == '\0')) {
		hDc = FindMonitor(screen);
		win_profile_len = MAX_PATH;
		win_default_profile[0] = '\0';
		SetICMMode(hDc, ICM_ON);
		if(GetICMProfileA(hDc, (LPDWORD) &win_profile_len, (LPSTR)win_default_profile))
		{
			if(strlen(win_default_profile) < 255)
				strcpy (in_name, win_default_profile);
		}
	}

	if(!donothing) {
		if(!hDc)
			hDc = FindMonitor(screen);
		if (clear) {
			if (!SetDeviceGammaRamp(hDc, &winGammaRamp))
				;//error ("Unable to reset display gamma");
			goto cleanupX;
		}
	}

	/* check for ramp size being a power of 2 and inside the supported range */
	switch(ramp_size)
	{
	case 16:
	case 32:
	case 64:
	case 128:
	case 256:
	case 512:
	case 1024:
	case 2048:
	case 4096:
	case 8192:
	case 16384:
	case 32768:
	case 65536:
		break;
	default:
		;//error("unsupported ramp size %u", ramp_size);
	}

	r_ramp = (unsigned short *) malloc (ramp_size * sizeof (unsigned short));
	g_ramp = (unsigned short *) malloc (ramp_size * sizeof (unsigned short));
	b_ramp = (unsigned short *) malloc (ramp_size * sizeof (unsigned short));

	if(!alter)
	{
		if( (i = read_vcgt_internal(in_name, r_ramp, g_ramp, b_ramp, ramp_size)) <= 0) {
			if(i<0)
				;//warning ("Unable to read file '%s'", in_name);
			if(i=0)
				;//warning ("No calibration data in ICC profile '%s' found", in_name);
			free(r_ramp);
			free(g_ramp);
			free(b_ramp);
			return false; //exit(0);
		}
	} else {
		static bool altern=true;
		if(altern)
			winGammaRamp.Blue[0]++;
		else
			winGammaRamp.Blue[0]--;
		altern = !altern;
		if (!GetDeviceGammaRamp(hDc, &winGammaRamp))
			;//warning ("Unable to get display calibration");

		for (i = 0; i < ramp_size; i++) {
			r_ramp[i] = winGammaRamp.Red[i];
			g_ramp[i] = winGammaRamp.Green[i];
			b_ramp[i] = winGammaRamp.Blue[i];
		}

	}

	{
		float redBrightness = 0.0;
		float redContrast = 100.0;
		float redMin = 0.0;
		float redMax = 1.0;
		float redGamma = 1.0;

		redMin = (double)r_ramp[0] / 65535.0;
		redMax = (double)r_ramp[ramp_size - 1] / 65535.0;
		redBrightness = redMin * 100.0;
		redContrast = (redMax - redMin) / (1.0 - redMin) * 100.0; 
		//message("Red Brightness: %f   Contrast: %f  Max: %f  Min: %f\n", redBrightness, redContrast, redMax, redMin);
	}

	{
		float greenBrightness = 0.0;
		float greenContrast = 100.0;
		float greenMin = 0.0;
		float greenMax = 1.0;
		float greenGamma = 1.0;

		greenMin = (double)g_ramp[0] / 65535.0;
		greenMax = (double)g_ramp[ramp_size - 1] / 65535.0;
		greenBrightness = greenMin * 100.0;
		greenContrast = (greenMax - greenMin) / (1.0 - greenMin) * 100.0; 
		//message("Green Brightness: %f   Contrast: %f  Max: %f  Min: %f\n", greenBrightness, greenContrast, greenMax, greenMin);
	}

	{
		float blueBrightness = 0.0;
		float blueContrast = 100.0;
		float blueMin = 0.0;
		float blueMax = 1.0;
		float blueGamma = 1.0;

		blueMin = (double)b_ramp[0] / 65535.0;
		blueMax = (double)b_ramp[ramp_size - 1] / 65535.0;
		blueBrightness = blueMin * 100.0;
		blueContrast = (blueMax - blueMin) / (1.0 - blueMin) * 100.0; 
		//message("Blue Brightness: %f   Contrast: %f  Max: %f  Min: %f\n", blueBrightness, blueContrast, blueMax, blueMin);
	}

	if(correction != 0)
	{
		for(i=0; i<ramp_size; i++)
		{
			r_ramp[i] =  65536.0 * (((double) pow (((double) r_ramp[i]/65536.0),
				xcalib_state.redGamma * (double) xcalib_state.gamma_cor
				) * (xcalib_state.redMax - xcalib_state.redMin)) + xcalib_state.redMin);
			g_ramp[i] =  65536.0 * (((double) pow (((double) g_ramp[i]/65536.0),
				xcalib_state.greenGamma * (double) xcalib_state.gamma_cor
				) * (xcalib_state.greenMax - xcalib_state.greenMin)) + xcalib_state.greenMin);
			b_ramp[i] =  65536.0 * (((double) pow (((double) b_ramp[i]/65536.0),
				xcalib_state.blueGamma * (double) xcalib_state.gamma_cor
				) * (xcalib_state.blueMax - xcalib_state.blueMin)) + xcalib_state.blueMin); 
		}
	}

	if(!invert) {
		/* ramps should be monotonic - otherwise content is nonsense! */
		for (i = 0; i < ramp_size - 1; i++) {
			if (r_ramp[i + 1] < r_ramp[i])
				;//warning ("red gamma table not monotonic");
			if (g_ramp[i + 1] < g_ramp[i])
				;//warning ("green gamma table not monotonic");
			if (b_ramp[i + 1] < b_ramp[i])
				;//warning ("blue gamma table not monotonic");
		}
	} else {
		for (i = 0; i < ramp_size; i++) {
			if(i >= ramp_size / 2)
				break;
			tmpRampVal = r_ramp[i];
			r_ramp[i] = r_ramp[ramp_size - i - 1];
			r_ramp[ramp_size - i - 1] = tmpRampVal;
			tmpRampVal = g_ramp[i];
			g_ramp[i] = g_ramp[ramp_size - i - 1];
			g_ramp[ramp_size - i - 1] = tmpRampVal;
			tmpRampVal = b_ramp[i];
			b_ramp[i] = b_ramp[ramp_size - i - 1];
			b_ramp[ramp_size - i - 1] = tmpRampVal;
		}
	}
	if(calcloss) {
		fprintf(stdout, "Resolution loss for %d entries:\n", ramp_size);
		r_res = 0;
		g_res = 0;
		b_res = 0;
		tmpRampVal = 0xffff;
		for(i = 0; i < ramp_size; i++) {
			if ((r_ramp[i] & 0xff00) != (tmpRampVal & 0xff00)) {
				r_res++;
			}
			tmpRampVal = r_ramp[i];
		}
		tmpRampVal = 0xffff;
		for(i = 0; i < ramp_size; i++) {
			if ((g_ramp[i] & 0xff00) != (tmpRampVal & 0xff00)) {
				g_res++;
			}
			tmpRampVal = g_ramp[i];
		}
		tmpRampVal = 0xffff;
		for(i = 0; i < ramp_size; i++) {
			if ((b_ramp[i] & 0xff00) != (tmpRampVal & 0xff00)) {
				b_res++;
			}
			tmpRampVal = b_ramp[i];
		}
		fprintf(stdout, "R: %d\tG: %d\t B: %d\t colors lost\n", ramp_size - r_res, ramp_size - g_res, ramp_size - b_res );
	}

	for (i = 0; i < ramp_size; i++) {
		winGammaRamp.Red[i] = r_ramp[i];
		winGammaRamp.Green[i] = g_ramp[i];
		winGammaRamp.Blue[i] = b_ramp[i];
	}

	if(printramps)
		for(i=0; i<ramp_size; i++)
			fprintf(stdout,"%x %x %x\n", r_ramp[i], g_ramp[i], b_ramp[i]);

	if(!donothing) {
		static bool altern=true;
		if(altern)
			winGammaRamp.Blue[127]++;
		else
			winGammaRamp.Blue[127]--;
		altern = !altern;
		if (!SetDeviceGammaRamp(hDc, &winGammaRamp))

			;//warning ("Unable to calibrate display");
	}

	free(r_ramp);
	free(g_ramp);
	free(b_ramp);

cleanupX:
	if(hDc)
		ReleaseDC(0, hDc); // gdi leak fix (goebish)
	return 0;
}
