#define VERSION "0.5"
#include <Windows.h>
#include "cpkeeper.h"
#include "calib.h"
#include <qmessagebox.h>
#include "settings.h"
#include <QFileDialog>
#include <shlobj.h>

CPKeeper::CPKeeper(QWidget *parent, Qt::WFlags flags)
	: QMainWindow(parent, flags)
{
	ui.setupUi(this);
	for(int i=0;i<32;i++)
	{
		monitorProfile[i]=NULL;
	}
	Apply = new QPushButton("Apply",this);
	Apply->setToolTip("Apply selected ICC profile(s) gamma ramp");
	connect(Apply, SIGNAL(clicked()), this, SLOT(applyClicked()));
	Lock = new QCheckBox("Lock profile(s)",this);
	Lock->setToolTip("Try to avoid other applications to override selected ICC profile gamma ramp");
	connect(Lock, SIGNAL(toggled(bool)), this, SLOT(lockToggled(bool)));
	About = new QPushButton("?",this);
	About->setToolTip("About");
	connect(About, SIGNAL(clicked()), this, SLOT(aboutClicked()));
	Reset = new QPushButton("Reset",this);
	Reset->setToolTip("Reset to default Windows ICC profile(s)");
	connect(Reset, SIGNAL(clicked()), this, SLOT(resetClicked()));
	Settings = new QLabel(this);
	Settings->setText ("<a href='The cake is a lie'><img src=':/CPKeeper/settings.png'></a>");  
	Settings->setToolTip("Settings");
	connect(Settings, SIGNAL(linkActivated(QString)), this, SLOT(settingsClicked()));
	Tray = new QSystemTrayIcon(this);
	Tray->setIcon( QIcon(":/CPKeeper/systray.png"));
	Tray->setToolTip("Color Profile Keeper");
	Tray->hide();
	connect(Tray, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(systrayClicked(QSystemTrayIcon::ActivationReason)));
	quitAction = new QAction("Quit",this);
	trayIconMenu = new QMenu(this);
	trayIconMenu->addAction(quitAction);
	Tray->setContextMenu(trayIconMenu);
	connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
	applyProfileTimer = new QTimer(this);
	applyProfileTimer->setInterval(3000);
	connect(applyProfileTimer, SIGNAL(timeout()), this, SLOT(applyProfileTimerTimeout()));
	loadSettings();
	init();
	connect(&selectProfileButtonGroup, SIGNAL(buttonClicked(QAbstractButton *)), this, SLOT(selectProfileButtonClicked(QAbstractButton *)));
	connect(&grabProfileButtonGroup, SIGNAL(buttonClicked(QAbstractButton *)), this, SLOT(grabProfileButtonClicked(QAbstractButton *)));
}

CPKeeper::~CPKeeper()
{
	saveSettings();
}

void CPKeeper::systrayClicked(QSystemTrayIcon::ActivationReason reason)
{
	if(reason==QSystemTrayIcon::Trigger)
	{
		showNormal();
		Tray->hide();
	}
}

void CPKeeper::changeEvent(QEvent *event)
{
	if (event->type() == QEvent::WindowStateChange)
	{
		QSettings _settings(QSettings::IniFormat, QSettings::UserScope, "Color Profile Keeper", "settings");
		if(_settings.value("tray_minimize","0").toString()=="1")
		{
			if(isMinimized())
			{
				qApp->processEvents();
				QTimer::singleShot(25, this, SLOT(hide()));
				event->ignore();
				setWindowFlags(windowFlags() | Qt::Tool);
				Tray->show();
				return;
			}
			else
			{
				show();
				setWindowFlags(windowFlags() & ~Qt::Tool);
			}
		}
	}
	QMainWindow::changeEvent(event);
}

void CPKeeper::closeEvent(QCloseEvent *event)
 {
	 QCoreApplication::quit();
 }

void CPKeeper::settingsClicked()
{
	settings settingsDialog(this);
	settingsDialog.exec();
}

void CPKeeper::resetClicked()
{
	Lock->setChecked(false);
	for(int index=0; index<m_monitorCount;index++)
		xcalib(index,"");
}

void CPKeeper::loadSettings()
{
	QSettings _settings(QSettings::IniFormat, QSettings::UserScope, "Color Profile Keeper", "settings");
	for(int index=0;index<32;index++)
	{
		QString file = _settings.value("monitor"+QString::number(index),"").toString();
		if(QFile::exists(file))
			monitorProfileFile[index] = file;
	}
	Lock->setChecked(_settings.value("lock","0").toString()=="1");
}

void CPKeeper::saveSettings()
{
	QSettings _settings(QSettings::IniFormat, QSettings::UserScope, "Color Profile Keeper", "settings");
	for(int index=0;index<32;index++)
	{
		if(monitorProfileFile[index]!="")
			_settings.setValue( "monitor"+QString::number(index), monitorProfileFile[index]);
	}
	_settings.setValue("lock", Lock->isChecked()?"1":"0");
	_settings.setValue("minimized", isMinimized()?"1":"0");
}

BOOL CALLBACK MonitorEnumProc2 (HMONITOR monitor, HDC hdc, LPRECT rect, LPARAM data)
{
	 int *Count = (int*)data;
    (*Count)++;
    return TRUE;
}

int CPKeeper::MonitorCount()
{
    int Count = 0;
    if (EnumDisplayMonitors(NULL, NULL, MonitorEnumProc2, (LPARAM)&Count))
        return Count;
    return -1;
}

void CPKeeper::aboutClicked()
{
	QMessageBox::information(this,"Color Profile Keeper","version " + QString(VERSION) + "\n(c)2013 Goebish\nlatest version available at\nhttp://goebish.free.fr/cpk\n\nthis tool allows to apply and lock\nICC profile files gamma ramp to\ntry to keep monitors calibration\neven while running or quitting\n3D fullscreen applications such\nas games");
}

void CPKeeper::selectProfileButtonClicked(QAbstractButton *button)
{
	int id =  selectProfileButtonGroup.id(button);
	TCHAR windir[MAX_PATH];
	GetWindowsDirectory(windir, MAX_PATH);
	QString file=QFileDialog::getOpenFileName(this,"Choose an ICC profile to use for monitor "+QString::number(id),QString::fromUtf16(windir)+"\\System32\\spool\\drivers\\color","ICC profile or CPKeeper gamma ramp(*.icc *.icm *.cpk)");
	if(QFile::exists(file))
	{
		monitorProfileFile[id]=file;
		monitorProfile[id]->setText(file);
	}
}

void CPKeeper::grabProfileButtonClicked(QAbstractButton *button)
{
	int id = grabProfileButtonGroup.id(button);
	TCHAR docdir[MAX_PATH];
	HRESULT result = SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, SHGFP_TYPE_CURRENT, docdir);
	QString file = QFileDialog::getSaveFileName(this,"Save monitor #" +QString::number(id) + " current gamma ramp as",QString::fromUtf16(docdir)+"/monitor "+QString::number(id),"CPKeeper gamma ramp(*.cpk)");
	if(file!="")
		grabAndSaveCurrentProfile(id,file);
}

void CPKeeper::applyProfile(int monitorID)
{
	if(QFile::exists(monitorProfileFile[monitorID]) && monitorID < m_monitorCount)
		xcalib(monitorID, monitorProfileFile[monitorID].toAscii().data());
}

void CPKeeper::applyClicked()
{
	for(int index=0;index<32;index++)
	{
		if(monitorLabel[index])
			applyProfile(index);
	}
}

void CPKeeper::lockToggled(bool checked)
{
	if(checked)
	{
		applyClicked();
		applyProfileTimer->start();
	}
	else
		applyProfileTimer->stop();
}

void CPKeeper::applyProfileTimerTimeout()
{
	applyClicked();
}

void CPKeeper::init()
{
	m_monitorCount = MonitorCount();
	int labelOffset=10;
	int grabOffset=65;
	int textOffset=83;
	int textlen=240;
	int currentYOffset = 20;
	if(m_monitorCount==-1)
	{
		QMessageBox::critical(this, "Error", "Couldn't enum monitor(s)");
		QApplication::exit(-1);
	}
	for(int index=0; index<m_monitorCount; index++)
	{
		monitorLabel[index] = new QLabel("Monitor " + QString::number(index), this);
		monitorLabel[index]->setGeometry( labelOffset, currentYOffset, 95, 15);
		
		grabProfileButton[index] = new QPushButton("*", this);
		grabProfileButton[index]->setToolTip("Grab & save monitor #"+ QString::number(index) + " current gamma ramp");
		grabProfileButton[index]->setGeometry( grabOffset, currentYOffset-2, 15, 20);
		grabProfileButtonGroup.addButton(grabProfileButton[index],index);
		grabProfileButtonGroup.setId(grabProfileButton[index],index);
	
		monitorProfile[index] = new QLineEdit(monitorProfileFile[index], this);
		monitorProfile[index]->setGeometry( textOffset, currentYOffset-2, textlen, 20);
		monitorProfile[index]->setText(monitorProfileFile[index]);
		
		selectProfileButton[index] = new QPushButton( "...", this);
		selectProfileButton[index]->setToolTip("Choose ICC profile to use for monitor #" + QString::number(index));
		selectProfileButton[index]->setGeometry( monitorProfile[index]->x()+monitorProfile[index]->width()+5, currentYOffset-2, 30 ,20);
		selectProfileButtonGroup.addButton(selectProfileButton[index],index);
		selectProfileButtonGroup.setId(selectProfileButton[index],index);
		
		currentYOffset+=30;
	}
	Lock->setGeometry(10, currentYOffset, 100, 20);
	Apply->setGeometry( 125, currentYOffset, 60, 20);
	Reset->setGeometry( 220, currentYOffset, 60, 20);
	About->setGeometry( width()-32, currentYOffset, 20,20);
	Settings->setGeometry( width()-56, currentYOffset, 20, 20);
	setMinimumHeight(currentYOffset+30);
	setMaximumHeight(currentYOffset+30);
	QSettings _settings(QSettings::IniFormat, QSettings::UserScope, "Color Profile Keeper", "settings");
	if(_settings.value("startup_apply","0").toString()=="1")
		applyClicked();
	if(_settings.value("minimized","0")=="1")
		showMinimized();
}
